#ifndef __PDP_AUX_H_INCLUDED__
#define __PDP_AUX_H_INCLUDED__

/**
@file
@brief Дополнительный функционал
*/

#include <parsec/pdp.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
@brief Безопасное создание каталога
@param dir имя создаваемого каталога
@param l метка
@param uid владелец
@param gid группа
@param mode режим доступа

@return 0 в случае успеха, в случае ошибки возвращается не 0 и выставляется errno
*/
int pdplugm_mkdir(const char *dir, const PDPL_T *l, uid_t uid, gid_t gid, mode_t mode);

/**
@brief Cоздание каталога для текущей сессии пользователя

Пример: /home/.pdp/user/l0i0c0x0t0x0/
@param root корневой каталог, в котором будут располагатся остальные каталоги, пример: /home/.pdp/
@param subdir необязательный подкаталог (может быть NULL или ""), в примере - user. Наследует метку с корневого каталога
@param slink имя символической сылки на subdir (если subdir не указан, то dir)
@param l указатель на метку сессии, преобразуется в текстовый вид и создается подкаталог с таким именем, на который устанавливается данная метка
@param uid владелец
@param gid группа
@param mode режим доступа

@return строку, содержащую полное имя созданного каталога, в примере /pdp_home/user/l0i0c0x0t0x0/, строка должна быть освобожденна вызовом
free(). В случае ошибки возвращается NULL и выставляется errno;

*/
char* pdplugm_make_session_dir(const char* root, const char* subdir,const char* slink, const PDPL_T *l,uid_t uid, gid_t gid, mode_t mode);

//------------------ remove this
char* pdp_make_session_dir(const char* root, const char* subdir,const char* slink, const PDPL_T *l,uid_t uid, gid_t gid, mode_t mode)  __attribute__ ((deprecated));
char* session_dir(const char* root, const char* subdir, const PDPL_T *l) __attribute__ ((deprecated));
//------------------ remove this

/**
@brief Формирование имени каталога для сессии пользователя

Пример /home/.pdp/user/l0i0c0x0t0x0/
@param root указатель на строку, не NULL, корневой каталог, в котором будут располагатся остальные каталоги, пример "/home/.pdp/".
Может быть пустой строкой "", в этом случае между между root и subdir будет добавлен '/'
@param subdir указатель на строку, не NULL, подкаталог, в котором будут располагатся остальные каталоги, пример "user".
Может быть пустой строкой "", в этом случае между между subdir и текстовым представлением l будет добавлен '/'
@param l указатель на метку сессии, преобразуется в текстовый вид и создается подкаталог с таким именем,
 на который устанавливается данная метка, не NULL

@return строку, содержащую полное имя каталога, в примере /pdp_home/user/l0i0c0x0t0x0/, строка должна быть освобожденна вызовом
free(). В случае ошибки возвращается NULL и выставляется errno
*/
char* pdpl_session_dir(const char* root, const char* subdir, const PDPL_T *l);

/**
@brief Формирование имени каталога для сессии пользователя

Выполняется с учетом только мандатной метки, пример: /home/.pdp/user/l0i0c0x0t0x0/
@param root указатель на строку, не NULL, корневой каталог, в котором будут располагатся остальные каталоги, пример "/home/.pdp/".
Может быть пустой строкой "", в этом случае между между root и subdir будет добавлен '/'
@param subdir указатель на строку, не NULL, подкаталог, в котором будут располагатся остальные каталоги, пример "user".
Может быть пустой строкой "", в этом случае между между subdir и текстовым представлением ml будет добавлен '/'
@param ml указатель на мандатную метку сессии, преобразуется в текстовый вид и создается подкаталог с таким именем,
 на который устанавливается данная метка, не NULL

@return строку, содержащую полное имя каталога, в примере: /pdp_home/user/l0i0c0x0t0x0/, строка должна быть освобожденна вызовом
free(). В случае ошибки возвращается NULL и выставляется errno
*/
char* pdpml_session_dir(const char* root, const char* subdir, const PDPML_T *ml);

#ifdef __cplusplus
}
#endif

#endif /* __PDP_AUX_H_INCLUDED__ */
